import streamlit as st
import matplotlib.pyplot as plt
from clusters import DBSCANCluster, KMeansCluster, CLUSTERING_STRATEGIES

st.header("Clustering")

if "data" in st.session_state:
    data = st.session_state.data

    general_row = st.columns([1, 1])
    clustering = general_row[0].selectbox("Clustering method", CLUSTERING_STRATEGIES)
    data_name = general_row[1].multiselect("Data Name",data.select_dtypes(include="number").columns, max_selections=3)

    with st.form("cluster_form"):
        if isinstance(clustering, KMeansCluster):
            row1 = st.columns([1, 1, 1])
            clustering.n_clusters = row1[0].number_input("Number of clusters", min_value=1, max_value=data.shape[0], value=clustering.n_clusters)
            clustering.n_init = row1[1].number_input("n_init", min_value=1, value=clustering.n_init)
            clustering.max_iter = row1[2].number_input("max_iter", min_value=1, value=clustering.max_iter)
        elif isinstance(clustering, DBSCANCluster):
            clustering.eps = st.slider("eps", min_value=0.0001, max_value=1.0, step=0.1, value=clustering.eps)
            clustering.min_samples = st.number_input("min_samples", min_value=1, value=clustering.min_samples)

        st.form_submit_button("Launch")

    if len(data_name) >= 2 and len(data_name) <=3:
        x = data[data_name].to_numpy()

        result = clustering.run(x)

        st.table(result.statistics)

        fig = plt.figure()
        if len(data_name) == 2:
            ax = fig.add_subplot(projection='rectilinear')
            plt.scatter(x[:, 0], x[:, 1], c=result.labels, s=50, cmap="viridis")
            if result.centers is not None:
                plt.scatter(result.centers[:, 0], result.centers[:, 1], c="black", s=200, marker="X")
        else:
            ax = fig.add_subplot(projection='3d')

            ax.scatter(x[:, 0], x[:, 1],x[:, 2], c=result.labels, s=50, cmap="viridis")
            if result.centers is not None:
                ax.scatter(result.centers[:, 0], result.centers[:, 1], result.centers[:, 2], c="black", s=200, marker="X")
        st.pyplot(fig)

else:
    st.error("file not loaded")
